@@ -18,7 +18,7 @@
 
 #ifdef WITH_TLS
 /**
- * $Id: 9142b91e2a7d390113314b7581ad2edd13aada1e $
+ * $Id: 62f57c4715a611284042edb28c284a7b91a6538b $
  *
  * @file tls.h
  * @brief Structures and prototypes for TLS wrappers
@@ -26,7 +26,7 @@
  * @copyright 2010 Network RADIUS SARL <info@networkradius.com>
  */
 
-RCSIDH(tls_h, "$Id: 9142b91e2a7d390113314b7581ad2edd13aada1e $")
+RCSIDH(tls_h, "$Id: 62f57c4715a611284042edb28c284a7b91a6538b $")
 
 #include <freeradius/conffile.h>
 
@@ -137,6 +137,8 @@ typedef struct _tls_session_t {
 	unsigned int 	(*record_minus)(record_t *buf, void *ptr, unsigned int size);
 
 	bool		invalid_hb_used;		//!< Whether heartbleed attack was detected.
+	bool		connected;			//!< whether the outgoing socket is connected
+	bool		is_init_finished;		//!< whether or not init is finished
 
 	/*
 	 *	Framed-MTU attribute in RADIUS, if present, can also be used to set this
@@ -293,8 +295,11 @@ void 		cbtls_msg(int write_p, int msg_version, int content_type, void const *buf
 			  void *arg);
 int		cbtls_verify(int ok, X509_STORE_CTX *ctx);
 
+/* threads.c */
+int		tls_mutexes_init(void);
+
 /* TLS */
-void		tls_global_init(void);
+int		tls_global_init(bool spawn_flag, bool check);
 #if 1
 int		tls_global_version_check(char const *acknowledged);
 #endif
@@ -305,9 +310,10 @@ int		tls_error_io_log(REQUEST *request, tls_session_t *session, int ret, char co
 
 void		tls_global_cleanup(void);
 tls_session_t	*tls_new_session(TALLOC_CTX *ctx, fr_tls_server_conf_t *conf, REQUEST *request, bool client_cert);
-tls_session_t	*tls_new_client_session(TALLOC_CTX *ctx, fr_tls_server_conf_t *conf, int fd);
+tls_session_t	*tls_new_client_session(TALLOC_CTX *ctx, fr_tls_server_conf_t *conf, int fd, VALUE_PAIR **certs);
 fr_tls_server_conf_t *tls_server_conf_parse(CONF_SECTION *cs);
 fr_tls_server_conf_t *tls_client_conf_parse(CONF_SECTION *cs);
+fr_tls_server_conf_t *tls_server_conf_alloc(TALLOC_CTX *ctx);
 SSL_CTX		*tls_init_ctx(fr_tls_server_conf_t *conf, int client);
 int 		tls_handshake_recv(REQUEST *, tls_session_t *ssn);
 int 		tls_handshake_send(REQUEST *, tls_session_t *ssn);
@@ -355,6 +361,9 @@ struct fr_tls_server_conf_t {
 	bool		disable_tlsv1_1;
 	bool		disable_tlsv1_2;
 
+	char const	*tls_min_version;
+	char const	*tls_max_version;
+
 	/*
 	 *	Always < 4096 (due to radius limit), 0 by default = 1024
 	 */
@@ -364,6 +373,7 @@ struct fr_tls_server_conf_t {
 	bool		allow_expired_crl;
 	char const	*check_cert_cn;
 	char const	*cipher_list;
+	bool		cipher_server_preference;
 	char const	*check_cert_issuer;
 
 	bool     	session_cache_enable;
@@ -371,8 +381,8 @@ struct fr_tls_server_conf_t {
 	uint32_t     	session_cache_size;
 	char const	*session_id_name;
 	char const	*session_cache_path;
+	fr_hash_table_t *cache_ht;
 	char		session_context_id[SSL_MAX_SSL_SESSION_ID_LENGTH];
-	time_t		session_last_flushed;
 
 	bool		verify_skip_if_ocsp_ok;
 	char const	*verify_tmp_dir;
